#!/bin/sh

MAILPLUS_SERVER_EVENT_LOG_CONF="/usr/local/etc/syslog-ng/patterndb.d/synolog/mailplus_server_event_log.conf"

reload()
{
	if [ -n "$(synoservice --status syslog-ng | grep "is start")" ]; then
		synoservice --reload syslog-ng
	fi
}

installEventLog()
{
	if [ -e "${MAILPLUS_SERVER_EVENT_LOG_CONF}" ]; then
		exit 0
	fi

	local targetLocation="/var/packages/MailPlus-Server/target"
	if [ ! -e "${targetLocation}" ]; then
		exit 1
	fi
	local volume=$(/usr/bin/readlink "${targetLocation}" | /usr/bin/cut -d'/' -f2)
	if ! grep -qsw "${volume}" /proc/mounts; then
		exit 1
	fi
	local tmpFileName="${MAILPLUS_SERVER_EVENT_LOG_CONF}.$$"

	echo -e "destination d_mailplus_server { file(\"/${volume}/@maillog/MailPlus_Server.log\" owner(\"MailPlus-Server\") group(\"system\") perm(0644) suppress(0) template(\"\$PRIORITY\\\t\$YEAR/\$MONTH/\$DAY \$HOUR:\$MIN:\$SEC\\\t\$MESSAGE\\\n\"));};
filter f_mailplus_server { program(\"^MailPlus-Server\") and level(warn..emerg); };
log { source(s_syno_syslog); filter(f_mailplus_server); destination(d_mailplus_server); };" > "${tmpFileName}"
	chmod 644 "${tmpFileName}"
	chown root:root "${tmpFileName}"
	mv "${tmpFileName}" "${MAILPLUS_SERVER_EVENT_LOG_CONF}"
}

uninstallEventLog()
{
	rm -f "${MAILPLUS_SERVER_EVENT_LOG_CONF}"
}

case "$1" in
	reload)
		reload
		;;
	install_event_log_conf)
		installEventLog
		;;
	uninstall_event_log_conf)
		uninstallEventLog
		;;
	check_event_log_conf_exist)
		[ -e "${MAILPLUS_SERVER_EVENT_LOG_CONF}" ]
		exit
		;;
	*)
		echo "Usage: $0 {reload|install_event_log_conf|uninstall_event_log_conf|event_log_conf_exist}" >&2
		exit 1
		;;
esac
